@@ -5,11 +5,10 @@ from pathlib import Path
 import click
 import dolfin
 
+from . import config
 from . import land_model
-from . import mechanics_model
 from . import postprocess as post
 from . import utils
-from .config import Config
 from .setup_models import Runner
 from .version import __version__
 
@@ -23,76 +22,67 @@ def cli():
 
 
 @click.command("run")
+@click.argument("geometry-path", type=click.Path(exists=True, resolve_path=True))
+@click.option(
+    "-s",
+    "--geometry-schema-path",
+    default=None,
+    help=(
+        "Schema for the geometry. If not provided it will assume that this file has the "
+        "same file name as `geometry-path` but with the suffix `.json`."
+    ),
+)
 @click.option(
     "-o",
     "--outdir",
-    default=Config.outdir,
+    default=config.Config.outdir,
     type=click.Path(writable=True, resolve_path=True),
     help="Output directory",
 )
-@click.option("--dt", default=Config.dt, type=float, help="Time step")
+@click.option("--dt", default=config.Config.dt, type=float, help="Time step")
 @click.option(
     "-T",
     "--end-time",
     "T",
-    default=Config.T,
+    default=config.Config.T,
     type=float,
     help="End-time of simulation",
 )
 @click.option(
     "-n",
     "--num_refinements",
-    default=Config.num_refinements,
+    default=config.Config.num_refinements,
     type=int,
     help="Number of refinements of for the mesh using in the EP model",
 )
 @click.option(
     "--save_freq",
-    default=Config.save_freq,
+    default=config.Config.save_freq,
     type=int,
     help="Set frequency of saving results to file",
 )
 @click.option(
     "--set_material",
-    default=Config.set_material,
+    default=config.Config.set_material,
     type=str,
     help="Choose material properties for mechanics model (default is HolzapfelOgden, option is Guccione",
 )
-@click.option("-dx", default=Config.dx, type=float, help="Spatial discretization")
 @click.option(
-    "-lx",
-    default=Config.lx,
-    type=float,
-    help="Size of mesh in x-direction",
-)
-@click.option(
-    "-ly",
-    default=Config.ly,
-    type=float,
-    help="Size of mesh in y-direction",
-)
-@click.option(
-    "-lz",
-    default=Config.lz,
-    type=float,
-    help="Size of mesh in z-direction",
-)
-@click.option(
-    "--bnd_cond",
-    default=Config.bnd_cond,
-    type=click.Choice(mechanics_model.BoundaryConditions._member_names_),
-    help="Boundary conditions for the mechanics problem",
+    "--bnd_rigid",
+    is_flag=True,
+    default=config.Config.bnd_rigid,
+    help="Flag to set boundary conditions for the mechanics problem to rigid motion condition",
 )
 @click.option(
     "--load_state",
     is_flag=True,
-    default=Config.load_state,
+    default=config.Config.load_state,
     help="If load existing state if exists, otherwise create a new state",
 )
 @click.option(
     "-IC",
     "--cell_init_file",
-    default=Config.cell_init_file,
+    default=config.Config.cell_init_file,
     type=str,
     help=(
         "Path to file containing initial conditions (json or h5 file). "
@@ -101,82 +91,79 @@ def cli():
 )
 @click.option(
     "--loglevel",
-    default=Config.loglevel,
+    default=config.Config.loglevel,
     type=int,
     help="How much printing. DEBUG: 10, INFO:20 (default), WARNING: 30",
 )
 @click.option(
-    "--hpc",
-    is_flag=True,
-    default=Config.hpc,
-    help="Indicate if simulations runs on hpc. This turns off the progress bar.",
+    "--show_progress_bar/--hide_progress_bar",
+    default=config.Config.show_progress_bar,
+    help="Shows or hide the progress bar.",
 )
 @click.option(
     "--drug_factors_file",
-    default=Config.drug_factors_file,
+    default=config.Config.drug_factors_file,
     type=str,
     help="Set drugs scaling factors (json file)",
 )
 @click.option(
     "--popu_factors_file",
-    default=Config.popu_factors_file,
+    default=config.Config.popu_factors_file,
     type=str,
     help="Set population scaling factors (json file)",
 )
 @click.option(
     "--disease_state",
-    default=Config.disease_state,
+    default=config.Config.disease_state,
     type=str,
     help="Indicate disease state. Default is healthy. ",
 )
 @click.option(
     "--mechanics-ode-scheme",
-    default=Config.mechanics_ode_scheme,
+    default=config.Config.mechanics_ode_scheme,
     type=click.Choice(land_model.Scheme._member_names_),
     help="Scheme used to solve the ODEs in the mechanics model",
 )
 @click.option(
     "--mechanics-use-continuation",
-    default=Config.mechanics_use_continuation,
+    default=config.Config.mechanics_use_continuation,
     type=bool,
     help="Use continuation based mechanics solver",
 )
 @click.option(
     "--mechanics-use-custom-newton-solver",
-    default=Config.mechanics_use_custom_newton_solver,
+    default=config.Config.mechanics_use_custom_newton_solver,
     type=bool,
     help="Use custom newton solver and solve ODEs at each Newton iteration",
 )
 @click.option(
     "--pcl",
-    default=Config.PCL,
+    default=config.Config.PCL,
     type=float,
     help="Pacing cycle length (ms)",
 )
 @click.option(
     "--spring",
-    default=Config.spring,
+    default=config.Config.spring,
     type=float,
     help="Set value of spring for Robin boundary condition",
 )
 @click.option(
     "--traction",
-    default=Config.traction,
+    default=config.Config.traction,
     type=float,
     help="Set value of traction for Neumann boundary condition",
 )
 def run(
+    geometry_path: utils.PathLike,
+    geometry_schema_path: typing.Optional[utils.PathLike],
     outdir: utils.PathLike,
     T: float,
-    dx: float,
     dt: float,
-    bnd_cond: mechanics_model.BoundaryConditions,
+    bnd_rigid: bool,
     load_state: bool,
     cell_init_file: utils.PathLike,
-    hpc: bool,
-    lx: float,
-    ly: float,
-    lz: float,
+    show_progress_bar: bool,
     save_freq: int,
     loglevel: int,
     num_refinements: int,
@@ -191,21 +178,18 @@ def run(
     spring: float,
     traction: float,
 ):
-
-    config = Config(
+    conf = config.Config(
+        geometry_path=geometry_path,
+        geometry_schema_path=geometry_schema_path,
         outdir=outdir,
         T=T,
-        dx=dx,
         dt=dt,
-        bnd_cond=bnd_cond,
+        bnd_rigid=bnd_rigid,
         spring=spring,
         traction=traction,
         load_state=load_state,
         cell_init_file=cell_init_file,
-        hpc=hpc,
-        lx=lx,
-        ly=ly,
-        lz=lz,
+        show_progress_bar=show_progress_bar,
         save_freq=save_freq,
         loglevel=loglevel,
         num_refinements=num_refinements,
@@ -218,7 +202,7 @@ def run(
         mechanics_use_custom_newton_solver=mechanics_use_custom_newton_solver,
         PCL=pcl,
     )
-    main(config=config)
+    main(conf=conf)
 
 
 @click.command("run-json")
@@ -227,30 +211,41 @@ def run_json(path):
     with open(path, "r") as json_file:
         data = json.load(json_file)
 
-    main(Config(**data))
+    main(config.Config(**data))
 
 
-def main(config: typing.Optional[Config]):
+def main(conf: typing.Optional[config.Config]):
 
-    if config is None:
-        config = Config()
+    if conf is None:
+        conf = config.Config()
+
+    geometry_path = Path(conf.geometry_path)
+    if not geometry_path.is_file():
+        msg = f"Unable to to find geometry path {geometry_path}"
+        raise IOError(msg)
+    if conf.geometry_schema_path is None:
+        conf.geometry_schema_path = geometry_path.with_suffix(".json")
 
     # Get all arguments and dump them to a json file
-    info_dict = config.__dict__
-    outdir = Path(config.outdir)
+    info_dict = conf.__dict__
+    outdir = Path(conf.outdir)
     outdir.mkdir(exist_ok=True)
     with open(outdir.joinpath("parameters.json"), "w") as f:
-        json.dump(info_dict, f)
+        json.dump(info_dict, f, default=post.json_serial)
 
     # Disable warnings
     from . import set_log_level
 
-    set_log_level(config.loglevel)
-    dolfin.set_log_level(config.loglevel + 10)  # TODO: Make it possible to set this?
+    set_log_level(conf.loglevel)
+    dolfin.set_log_level(conf.loglevel + 10)  # TODO: Make it possible to set this?
 
-    runner = Runner(config=config)
+    runner = Runner(conf=conf)
 
-    runner.solve(T=config.T, save_freq=config.save_freq, hpc=config.hpc)
+    runner.solve(
+        T=conf.T,
+        save_freq=conf.save_freq,
+        show_progress_bar=conf.show_progress_bar,
+    )
 
 
 @click.command("postprocess")
@@ -273,11 +268,23 @@ def main(config: typing.Optional[Config]):
     default=False,
     help="Plot population",
 )
+@click.option(
+    "--reduction",
+    default="average",
+    help="What type of reduction to perform when plotting state traces, by default 'average'.",
+)
 @click.option("--num_models", default=5, help="Number of models to be analyzed")
-def postprocess(folder, num_models, plot_state_traces, make_xdmf, population):
+def postprocess(
+    folder,
+    num_models,
+    plot_state_traces,
+    make_xdmf,
+    population,
+    reduction,
+):
     folder = Path(folder)
     if plot_state_traces:
-        post.plot_state_traces(folder.joinpath("results.h5"))
+        post.plot_state_traces(folder.joinpath("results.h5"), reduction=reduction)
     if make_xdmf:
         post.make_xdmffiles(folder.joinpath("results.h5"))
     if population:
