@@ -5,25 +5,20 @@ from dataclasses import dataclass
 import dolfin
 
 from . import land_model
-from . import mechanics_model
 from . import utils
 
 
 @dataclass
 class Config:
     outdir: utils.PathLike = "results"
+    geometry_path: utils.PathLike = ""
+    geometry_schema_path: typing.Optional[utils.PathLike] = None
     T: float = 1000
-    dx: float = 0.2
     dt: float = 0.05
-    bnd_cond: mechanics_model.BoundaryConditions = (
-        mechanics_model.BoundaryConditions.dirichlet
-    )
+    bnd_rigid: bool = False
     load_state: bool = False
     cell_init_file: utils.PathLike = ""
-    hpc: bool = False
-    lx: float = 2.0
-    ly: float = 0.7
-    lz: float = 0.3
+    show_progress_bar: bool = True
     save_freq: int = 1
     pre_stretch: typing.Optional[typing.Union[dolfin.Constant, float]] = None
     traction: typing.Union[dolfin.Constant, float] = None
